//+------------------------------------------------------------------+
//|                                                  EmailStatus.mq4 |
//|                                               Paul Hampton-Smith |
//|                                                                  |
//+------------------------------------------------------------------+

#include <TimeGMT.mqh>

#property indicator_chart_window
//---- input parameters
extern int       EmailHourStart = 9; // local time
extern int       EmailHourEnd  = 18;
extern string    Subject = "test email";

datetime g_dtLastReport = 0;

string g_strCountries[] = {"USD","GBP","EUR","CHF","CAD","JPY","AUD","NZD","MXN","XAU","XAG","TRY"};
string g_strSymbols[];

#include <stdlib.mqh>


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	// set last report time as 3 days ago to cover weekends and additional history first time
	g_dtLastReport = TimeLocalEx()-3*24*3600;
	
	int nSymbolCount = 0;
	for (int i = 0 ; i < ArraySize(g_strCountries) ; i++)
	{
		for (int j = i+1 ; j < ArraySize(g_strCountries) ; j++)
		{
			if (PairExists(g_strCountries[i]+g_strCountries[j]))
			{ 
				ArrayResize(g_strSymbols,nSymbolCount+1); 
				g_strSymbols[nSymbolCount] = g_strCountries[i]+g_strCountries[j];
				nSymbolCount++;
			}
			if (PairExists(g_strCountries[j]+g_strCountries[i]))
			{ 
				ArrayResize(g_strSymbols,nSymbolCount+1); 
				g_strSymbols[nSymbolCount] = g_strCountries[j]+g_strCountries[i];
				nSymbolCount++;
			}
		}
	}

	start();
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
	static int nLastHourLocal = -1;
	int nHourLocal = TimeHour(TimeLocalEx());
	if (nHourLocal != nLastHourLocal &&
		nHourLocal >= EmailHourStart && 
		nHourLocal <= EmailHourEnd) 
	{
		if (EmailStatus()) nLastHourLocal	= nHourLocal;
	}
	return(0);
}
//+------------------------------------------------------------------+

bool EmailStatus()
{
	double dblOpenProfit = 0.0;
	for (int i = 0 ; i < OrdersTotal() ; i++)
	{
		OrderSelect(i, SELECT_BY_POS);
		dblOpenProfit += OrderProfit() + OrderSwap();
	}
	string strBody = StringConcatenate(
		  "Balance     ",RightJustifyDblToStr(AccountBalance(),6,0),
		"\nEquity      ",RightJustifyDblToStr(AccountEquity(),6,0),
		"\nMargin      ",RightJustifyDblToStr(AccountMargin(),6,0),
		"\nFree margin ",RightJustifyDblToStr(AccountFreeMargin(),6,0),
		"\nOpen profit ",RightJustifyDblToStr(dblOpenProfit,6,0),"\n\n");

	if (OrdersTotal() == 0)
	{
		strBody = strBody + "No open or pending orders";
	}
	else
	{
		strBody = strBody + OpenOrderReport();
		strBody = strBody + "\n\n" + PendingOrderReport();
	}

	strBody = strBody + "\n\n" + ClosedOrdersSinceLastReport();
	GetLastError();
	SendMail(Subject, strBody);
	return(GetLastError() == 0);
}	

string OpenOrderReport()
{
	string strReport = "Open orders\n\n";

	if (StringLen(Symbol())==6)
	{
		strReport = strReport + "Open time local Type Lots Symbol  Price   S/L    T/P    Now Profit Comment\n";
		strReport = strReport + "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~";
	}
	else
	{
		strReport = strReport + "Open time local Type Lots  Symbol  Price   S/L    T/P    Now Profit Comment\n";
		strReport = strReport + "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~";
	}

	for (int j = 0 ; j < ArraySize(g_strSymbols) ; j++)
	{
		for (int i = 0 ; i < OrdersTotal() ; i++)
		{
			OrderSelect(i, SELECT_BY_POS);
			if (OrderSymbol() != g_strSymbols[j]) continue;
			if (OrderType() > OP_SELL) continue;
			double dblDigits = MarketInfo(OrderSymbol(),MODE_DIGITS);
			strReport = StringConcatenate(strReport,"\n",
				ShortTimeToStr(ServerToLocal(OrderOpenTime()))," ",
				BuySellToStrFixed(OrderType())," ",
				RightJustifyLotStr(OrderSymbol(), OrderLots(),4)," ",
				OrderSymbol()," ", 
				RightJustifyDblToStr(OrderOpenPrice(),6,dblDigits)," ",
				RightJustifyDblToStr(OrderStopLoss(),6,dblDigits)," ",
				RightJustifyDblToStr(OrderTakeProfit(),6,dblDigits)," ", 
				RightJustifyDblToStr(MarketInfo(OrderSymbol(),MODE_BID),6,dblDigits)," ", 
				RightJustifyDblToStr(OrderProfit()+OrderSwap(),5,0)," ",
				OrderComment());
		}
	}
	return(strReport);
}			

string PendingOrderReport()
{
	string strReport = "Pending orders\n\n";

	if (StringLen(Symbol())==6)
	{
		strReport = strReport + "Open time local      Type Lots Symbol  Price   S/L    T/P    Now  Comment\n";
		strReport = strReport + "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~";
	}
	else
	{
		strReport = strReport + "Open time local      Type Lots  Symbol  Price   S/L    T/P    Now  Comment\n";
		strReport = strReport + "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~";
	}
	
	for (int j = 0 ; j < ArraySize(g_strSymbols) ; j++)
	{
		for (int i = 0 ; i < OrdersTotal() ; i++)
		{
			OrderSelect(i, SELECT_BY_POS);
			if (OrderSymbol() != g_strSymbols[j]) continue;
			if (OrderType() <= OP_SELL) continue;
			double dblDigits = MarketInfo(OrderSymbol(),MODE_DIGITS);
			strReport = StringConcatenate(strReport,"\n",
				ShortTimeToStr(ServerToLocal(OrderOpenTime()))," ",
				CmdToStrFixed(OrderType())," ",
				RightJustifyLotStr(OrderSymbol(), OrderLots(),4)," ",
				OrderSymbol()," ", 
				RightJustifyDblToStr(OrderOpenPrice(),6,dblDigits)," ",
				RightJustifyDblToStr(OrderStopLoss(),6,dblDigits)," ",
				RightJustifyDblToStr(OrderTakeProfit(),6,dblDigits)," ", 
				RightJustifyDblToStr(MarketInfo(OrderSymbol(),MODE_BID),6,dblDigits)," ", 
				OrderComment());
		}
	}
	return(strReport);
}			

string ClosedOrdersSinceLastReport()
{
	string strReport = "";
	
	for (int i = OrdersHistoryTotal()-1 ; i >= 0 ; i--)
	{
		OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
		// only orders since last report
		if (ServerToLocal(OrderCloseTime()) < g_dtLastReport ) continue;

		double dblDigits = MarketInfo(OrderSymbol(),MODE_DIGITS);
		strReport = StringConcatenate(strReport,"\n",
			ShortTimeToStr(ServerToLocal(OrderOpenTime()))," ",
			CmdToStrFixed(OrderType())," ",
			RightJustifyLotStr(OrderSymbol(), OrderLots(),4)," ",
			OrderSymbol()," ", 
			RightJustifyDblToStr(OrderOpenPrice(),6,dblDigits)," ",
			RightJustifyDblToStr(OrderStopLoss(),6,dblDigits)," ",
			RightJustifyDblToStr(OrderTakeProfit(),6,dblDigits)," ", 
			ShortTimeToStr(ServerToLocal(OrderCloseTime()))," ",
			RightJustifyDblToStr(OrderClosePrice(),6,dblDigits)," ", 
			RightJustifyDblToStr(OrderProfit()+OrderSwap(),5,0)," ",
			OrderComment());
	}
	
	if (strReport == "")
	{
		return("No closed orders since local time " + TimeToStr(g_dtLastReport,TIME_DATE|TIME_SECONDS));
	}
	else
	{
		if (StringLen(Symbol())==6)
		{
			strReport = StringConcatenate(
				"Orders closed since local time ",TimeToStr(g_dtLastReport,TIME_DATE|TIME_SECONDS),
				"\n\n",
				"Open time local      Type Lots Symbol  Price   S/L    T/P  Close time local Close Profit Comment\n",
				"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~",
				strReport);
		}
		else
		{
			strReport = StringConcatenate(
				"Orders closed since local time ",TimeToStr(g_dtLastReport,TIME_DATE|TIME_SECONDS),
				"\n\n",
				"  Open time local         Type  Lots  Symbol  Price   S/L    T/P   Close time local   Close Profit Comment\n",
				"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~",
				strReport);
		}
		
		g_dtLastReport = TimeLocalEx();
		return(strReport);
	}
}			


string CmdToStrFixed(int cmd)
{
   switch(cmd)
   {
   case OP_BUY:       return("      Buy");
   case OP_SELL:      return("     Sell");
   case OP_BUYSTOP:   return("  BuyStop");
   case OP_SELLSTOP:  return(" SellStop");
   case OP_BUYLIMIT:  return(" BuyLimit");
   case OP_SELLLIMIT: return("SellLimit");
   default: return("        ");
   }
}      

string BuySellToStrFixed(int cmd)
{
   switch(cmd)
   {
   case OP_BUY:       return(" Buy");
   case OP_SELL:      return("Sell");
   default: return("    ");
   }
}      

	
string RightJustifyDblToStr(double dbl, int nWidth, int nDecimals) 
{ 
	string strRet = DoubleToStr(dbl,nDecimals); 
   int nLen = StringLen(strRet); 
   for (int i = nLen ; i < nWidth ; i++) strRet = " " + strRet; 
   return(strRet); 
} 
        
string RightJustifyLotStr(string strSymbol, double dblLots, int nWidth)
{
	double dblMinLot = NormalizeDouble(MarketInfo(strSymbol,MODE_MINLOT),2);
	if (dblMinLot == 0.01) return(RightJustifyDblToStr(dblLots, nWidth, 2));
	if (dblMinLot == 0.1)  return(RightJustifyDblToStr(dblLots, nWidth, 1));
}
	
string ShortTimeToStr(datetime dt)
{
	string strMonth;
	switch (TimeMonth(dt))
	{
	case  1: strMonth = "Jan"; break;
	case  2: strMonth = "Feb"; break;
	case  3: strMonth = "Mar"; break;
	case  4: strMonth = "Apr"; break;
	case  5: strMonth = "May"; break;
	case  6: strMonth = "Jun"; break;
	case  7: strMonth = "Jul"; break;
	case  8: strMonth = "Aug"; break;
	case  9: strMonth = "Sep"; break;
	case 10: strMonth = "Oct"; break;
	case 11: strMonth = "Nov"; break;
	case 12: strMonth = "Dec"; break;
	default: strMonth = "???";
	}
	return(Pad(TimeDay(dt)," ")+" "+strMonth+" "+Pad(TimeHour(dt)," ")+":"+Pad(TimeMinute(dt),"0")+":"+Pad(TimeSeconds(dt),"0"));
}
	
string Pad(int n, string strPadChar)
{
	if (n <= 9) 
		return(strPadChar+n);
	else
		return(n);
}

bool PairExists(string strPair)
{
	return(MarketInfo(strPair,MODE_DIGITS) != 0);
}
	